package com.huifu.jh.core.message;

import com.alibaba.fastjson.JSON;
import com.huifu.jh.core.model.GcbPayResponse;
import com.huifu.jh.core.model.ParamResponse;
import com.huifu.jh.core.util.UrlCoderUtil;
import com.huifu.jh.core.util.dto.GcbPayConstants;
import com.huifu.jh.core.util.dto.HttpResp;

import java.net.URLDecoder;
import java.util.HashMap;
import java.util.Map;

/**
 * @author fang.wei
 * @Description 参数格式响应解析器
 * @createTime 2021年10月17日 22:11:00
 */
public class ParamStringResponseParser <T extends GcbPayResponse> implements ResponseParser<T> {

    private Class<T> clazz;

    public ParamStringResponseParser(Class<T> clazz) {
        this.clazz = clazz;
    }

    @Override
    public T parse(HttpResp httpResp) throws Exception {
    	String responseTxt = new String(httpResp.getResponseBody(), GcbPayConstants.ENCODING);
    	
        Map<String,String> map = new HashMap<>();
        String[] pairArray = responseTxt.split("&");
        for (String pair : pairArray) {
            String[] pv = pair.split("=", 2);
            // URL解码处理参数值
            if (UrlCoderUtil.hasUrlEncoded(pv[1])) {
            	map.put(pv[0], URLDecoder.decode(pv[1], GcbPayConstants.ENCODING));
            } else {
            	map.put(pv[0], pv[1]);
            }
            
        }

        String jsonStr = JSON.toJSONString(map);
        System.out.println("Response: " + jsonStr);
        T response = JSON.parseObject(jsonStr, clazz);
        response.setHttpStatus(httpResp.getHttpStatus());
        response.setResponseBody(httpResp.getResponseBody());
        
        ParamResponse paramResponse = (ParamResponse) response;
        paramResponse.setParamMap(map);
        return response;
    }
}