package com.huifu.jh.core.util;

import com.huifu.jh.core.util.dto.HttpResp;
import org.apache.http.HttpEntity;
import org.apache.http.NameValuePair;
import org.apache.http.client.config.AuthSchemes;
import org.apache.http.client.config.CookieSpecs;
import org.apache.http.client.config.RequestConfig;
import org.apache.http.client.entity.UrlEncodedFormEntity;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.config.Registry;
import org.apache.http.config.RegistryBuilder;
import org.apache.http.conn.socket.ConnectionSocketFactory;
import org.apache.http.conn.socket.PlainConnectionSocketFactory;
import org.apache.http.conn.ssl.NoopHostnameVerifier;
import org.apache.http.conn.ssl.SSLConnectionSocketFactory;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.impl.conn.PoolingHttpClientConnectionManager;
import org.apache.http.message.BasicNameValuePair;
import org.apache.http.util.EntityUtils;

import javax.net.ssl.SSLContext;
import javax.net.ssl.TrustManager;
import javax.net.ssl.X509TrustManager;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.security.KeyManagementException;
import java.security.NoSuchAlgorithmException;
import java.security.cert.X509Certificate;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Map;

/**
 * http客户端
 */
public class HttpsClientUtils {
	private static final int SOCKET_TIMEOUT 	= 60 * 1000; // 60秒
	private static final int CONNECTION_TIMEOUT = 60 * 1000; // 60秒
	
	private static final RequestConfig REQUEST_CONFIG = RequestConfig.custom()
														.setSocketTimeout(SOCKET_TIMEOUT)
														.setConnectTimeout(CONNECTION_TIMEOUT).build();

	private static CloseableHttpClient httpclient = createSSLClient();
	
	/**
	 * form post
	 * @param url
	 * @param params
	 * @return
	 * @throws Exception
	 */
	public static HttpResp postRequest(String url, Map<String, String> params, String charset) throws Exception {
		HttpPost httpPost = new HttpPost(url);  
		httpPost.setConfig(REQUEST_CONFIG);

		// 设置数据
        HttpEntity entity = buildHttpEntity(params, charset);
		httpPost.setEntity(entity);
		
		CloseableHttpResponse response = null;
		try {
			response = httpclient.execute(httpPost);  

			HttpResp resp = new HttpResp();
			resp.setHttpStatus(response.getStatusLine().getStatusCode());
			resp.setResponseBody(EntityUtils.toByteArray(response.getEntity()));

			if (resp.getResponseBody() == null || resp.getResponseBody().length == 0) {
				throw new RuntimeException("Empty response");
			}

			// set contentType
			if (response.getEntity().getContentType() != null) {
				resp.setContentType(response.getEntity().getContentType().getValue());
			}
			return resp;
		} finally {  
			closeHttpResponse(response);
		}
	}

	/**
	 * xml post
	 * @param url
	 * @param xml
	 * @param charset
	 * @return
	 * @throws Exception
	 */
	public static HttpResp postXml(String url, String xml, String charset) throws Exception {
		HttpPost httpPost = new HttpPost(url);
		httpPost.setConfig(REQUEST_CONFIG);
		httpPost.setHeader("Content-Type", "text/xml; charset=UTF-8");

		// 设置数据
		StringEntity entity = new StringEntity(xml, charset);
		httpPost.setEntity(entity);

		CloseableHttpResponse response = null;
		try {
			response = httpclient.execute(httpPost);

			HttpResp resp = new HttpResp();
			resp.setHttpStatus(response.getStatusLine().getStatusCode());
			resp.setResponseBody(EntityUtils.toByteArray(response.getEntity()));
			resp.setContentType(response.getEntity().getContentType().getValue());
			return resp;
		} finally {
			closeHttpResponse(response);
		}
	}

	/**
	 * 
	 * @param params
	 * @return
	 * @throws UnsupportedEncodingException 
	 */
	private static HttpEntity buildHttpEntity(Map<String, String> params, String charset) throws UnsupportedEncodingException {

		List <NameValuePair> nvps = new ArrayList <NameValuePair>();
		if (params != null) {
			for (Map.Entry<String, String> entry : params.entrySet()) {
				
				// 过滤空值
				if (entry.getValue() == null || entry.getValue().trim().equals("")) {
					continue;
				}
				
				nvps.add(new BasicNameValuePair(entry.getKey(), entry.getValue().toString()));
			}
		}

        return new UrlEncodedFormEntity(nvps, charset);
	}
	
	/**
	 * 
	* @Title: closeHttpResponse
	* @Description: 关闭http response
	* @param httpResponse    void
	* @throws
	 */
	private static void closeHttpResponse(CloseableHttpResponse httpResponse) {
		if (httpResponse == null) {
			return;
		}

		try {
			httpResponse.close();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
	
	/**
	 * 
	* @Title: createSSLClient
	* @Description: 创建https客户端
	* @return    CloseableHttpClient
	* @throws
	 */
    private static CloseableHttpClient createSSLClient() {
        try {
            // 在调用SSL之前需要重写验证方法，取消检测SSL
            X509TrustManager trustManager = new X509TrustManager() {
                @Override public X509Certificate[] getAcceptedIssuers() {
                    return null;
                }
                @Override public void checkClientTrusted(X509Certificate[] xcs, String str) {}
                @Override public void checkServerTrusted(X509Certificate[] xcs, String str) {}
            };
            SSLContext ctx = SSLContext.getInstance(SSLConnectionSocketFactory.TLS);
            ctx.init(null, new TrustManager[] { trustManager }, null);
            SSLConnectionSocketFactory socketFactory = new SSLConnectionSocketFactory(ctx, NoopHostnameVerifier.INSTANCE);
            // 创建Registry
            RequestConfig requestConfig = RequestConfig.custom().setCookieSpec(CookieSpecs.STANDARD_STRICT)
                    .setExpectContinueEnabled(Boolean.TRUE).setTargetPreferredAuthSchemes(Arrays.asList(AuthSchemes.NTLM,AuthSchemes.DIGEST))
                    .setProxyPreferredAuthSchemes(Arrays.asList(AuthSchemes.BASIC)).build();
            Registry<ConnectionSocketFactory> socketFactoryRegistry = RegistryBuilder.<ConnectionSocketFactory>create()
                    .register("http", PlainConnectionSocketFactory.INSTANCE)
                    .register("https",socketFactory).build();
            // 创建ConnectionManager，添加Connection配置信息
            PoolingHttpClientConnectionManager connectionManager = new PoolingHttpClientConnectionManager(socketFactoryRegistry);
            connectionManager.setMaxTotal(100);
            connectionManager.setDefaultMaxPerRoute(50);
            
            CloseableHttpClient closeableHttpClient = HttpClients.custom().setConnectionManager(connectionManager)
                    .setDefaultRequestConfig(requestConfig).build();
            return closeableHttpClient;
        } catch (KeyManagementException ex) {
            throw new RuntimeException(ex);
        } catch (NoSuchAlgorithmException ex) {
            throw new RuntimeException(ex);
        }
    }

}