package com.huifu.jh.core.util;

import java.security.PrivateKey;
import java.security.PublicKey;
import java.util.Collections;

import javax.xml.crypto.dsig.CanonicalizationMethod;
import javax.xml.crypto.dsig.DigestMethod;
import javax.xml.crypto.dsig.Reference;
import javax.xml.crypto.dsig.SignatureMethod;
import javax.xml.crypto.dsig.SignedInfo;
import javax.xml.crypto.dsig.Transform;
import javax.xml.crypto.dsig.XMLSignature;
import javax.xml.crypto.dsig.XMLSignatureFactory;
import javax.xml.crypto.dsig.dom.DOMSignContext;
import javax.xml.crypto.dsig.dom.DOMValidateContext;
import javax.xml.crypto.dsig.spec.C14NMethodParameterSpec;
import javax.xml.crypto.dsig.spec.TransformParameterSpec;

import com.huifu.jh.core.exception.GcbPayException;
import org.w3c.dom.Document;
import org.w3c.dom.NodeList;

/**
 * XML安全工具类
 */
public class XMLSecurityUtils {

	/**
	 * xml加签
	 * @param doc
	 * @param privateKeyString
	 * @return
	 * @throws GcbPayException
	 */
	public static Document xmlSignature(Document doc, String privateKeyString) throws Exception {
		XMLSignatureFactory factory = XMLSignatureFactory.getInstance("DOM");

		// Create a Reference to the enveloped document (in this case we are
		// signing the whole document, so a URI of "" signifies that) and
		// also specify the SHA1 digest algorithm and the ENVELOPED Transform.
		Reference ref = factory.newReference("", factory.newDigestMethod(
		DigestMethod.SHA1, null), Collections.singletonList(factory.newTransform(Transform.ENVELOPED,
		(TransformParameterSpec) null)), null, null);

		// Create the SignedInfo
		SignedInfo si = factory.newSignedInfo(factory.newCanonicalizationMethod(
		CanonicalizationMethod.INCLUSIVE_WITH_COMMENTS,
		(C14NMethodParameterSpec) null), factory.newSignatureMethod(SignatureMethod.RSA_SHA1, null),
		Collections.singletonList(ref));

		PrivateKey privateKey = SecretKeyUtils.transform2PrivateKey(privateKeyString);

		// Create a DOMSignContext and specify the DSA PrivateKey and
		// location of the resulting XMLSignature's parent element
		DOMSignContext dsc = new DOMSignContext(privateKey, doc.getDocumentElement());

		// Create the XMLSignature (but don't sign it yet)
		XMLSignature signature = factory.newXMLSignature(si, null);

		// Marshal, generate (and sign) the enveloped signature
		signature.sign(dsc);

		// output the resulting document
		return doc;
	}

	/**
	 * xml验签
	 * @param doc
	 * @param publicKeyString
	 * @return
	 * @throws Exception
	 */
	public static boolean verifySignature(Document doc, String publicKeyString) throws Exception {
		NodeList nl = doc.getElementsByTagNameNS(XMLSignature.XMLNS, "Signature");
		if (nl.getLength() == 0) {
			throw new RuntimeException("Cannot find Signature element");
		}

		XMLSignatureFactory factory = XMLSignatureFactory.getInstance("DOM");
		PublicKey publicKey = SecretKeyUtils.transform2PublicKey(publicKeyString);
		DOMValidateContext valContext = new DOMValidateContext(publicKey, nl.item(0));
		XMLSignature signature = factory.unmarshalXMLSignature(valContext);
		boolean coreValidity = signature.validate(valContext);
		return coreValidity;
	}

}
