package com.huifu.jh.payout.model;

import com.huifu.jh.core.model.ParamRequest;
import org.apache.commons.lang.StringUtils;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author fang.wei
 * @Description 余额支付请求
 * @createTime 2021年10月28日 10:52:00
 */
public class GcbAccTransferApplyRequest implements ParamRequest<GcbAccTransferApplyResponse> {

    /**
     * 网关版本 固定值：1
     * 必填
     */
    private String version = "1";

    /**
     * 签名类型 固定值：4。RSA加签
     * 必填
     */
    private String signType = "4";

    /**
     * 请求时间 yyyyMMddHHmmss
     * 必填
     */
    private String requestTime;

    /**
     * 商户会员号 由我司提供（即11位商户号+01）例如商户号为10012021109，则此处填写1001202110901
     * 必填
     */
    private String merchantAcctId;

    /**
     * 终端号 由我司提供
     * 必填
     */
    private String terminalId;

    /**
     * 批次号 商户系统中唯一标识该笔批次，失败重发请更换批次号
     * 必填
     */
    private String batchNo;

    /**
     * 付款币种 默认CNY
     * 必填
     */
    private String payCur = "CNY";

    /**
     * 总笔数 -
     * 必填
     */
    private String payTCnt;

    /**
     * 总金额 单位分，明细总金额之和
     * 必填
     */
    private String payTAmt;

    /**
     * 扩展字段1 备用
     */
    private String ext1;

    /**
     * 扩展字段2 备用
     */
    private String ext2;

    /**
     * 结果通知地址 备用
     * 必填
     */
    private String noticeUrl;

    /**
     * 重复拦截校验标志 重复拦截标志，默认值为1校验 0-不校验 1- 校验
     * 校验规则：
     * 批次明细按以下格式拼接，当日存在完全相同的交易则拦截不允许交易
     * 付款人类型+付款人账号+收款人类型+收款人账号+币别+金额
     */
    private String dupChkFlag;

    /**
     * 支付明细
     */
    private List<GcbAccTransferApplyRequestDetail> details;

    public String getVersion() {
        return version;
    }

    public void setVersion(String version) {
        this.version = version;
    }

    public String getSignType() {
        return signType;
    }

    public void setSignType(String signType) {
        this.signType = signType;
    }

    public String getRequestTime() {
        return requestTime;
    }

    public void setRequestTime(String requestTime) {
        this.requestTime = requestTime;
    }

    public String getMerchantAcctId() {
        return merchantAcctId;
    }

    public void setMerchantAcctId(String merchantAcctId) {
        this.merchantAcctId = merchantAcctId;
    }

    public String getTerminalId() {
        return terminalId;
    }

    public void setTerminalId(String terminalId) {
        this.terminalId = terminalId;
    }

    public String getBatchNo() {
        return batchNo;
    }

    public void setBatchNo(String batchNo) {
        this.batchNo = batchNo;
    }

    public String getPayCur() {
        return payCur;
    }

    public void setPayCur(String payCur) {
        this.payCur = payCur;
    }

    public String getPayTCnt() {
        return payTCnt;
    }

    public void setPayTCnt(String payTCnt) {
        this.payTCnt = payTCnt;
    }

    public String getPayTAmt() {
        return payTAmt;
    }

    public void setPayTAmt(String payTAmt) {
        this.payTAmt = payTAmt;
    }

    public String getExt1() {
        return ext1;
    }

    public void setExt1(String ext1) {
        this.ext1 = ext1;
    }

    public String getExt2() {
        return ext2;
    }

    public void setExt2(String ext2) {
        this.ext2 = ext2;
    }

    public String getNoticeUrl() {
        return noticeUrl;
    }

    public void setNoticeUrl(String noticeUrl) {
        this.noticeUrl = noticeUrl;
    }

    public String getDupChkFlag() {
        return dupChkFlag;
    }

    public void setDupChkFlag(String dupChkFlag) {
        this.dupChkFlag = dupChkFlag;
    }

    public List<GcbAccTransferApplyRequestDetail> getDetails() {
        return details;
    }

    public void setDetails(List<GcbAccTransferApplyRequestDetail> details) {
        this.details = details;
    }

    @Override
    public String getMerchantId() {
        return merchantAcctId;
    }

    @Override
    public Class<GcbAccTransferApplyResponse> getResponseClass() {
        return GcbAccTransferApplyResponse.class;
    }

    @Override
    public String[] getSignParamNames() {
        return new String[] {"version","signType","requestTime","merchantAcctId","terminalId","batchNo","payCur","payTCnt","payTAmt","ext1","ext2","noticeUrl","details","dupChkFlag"};
    }

    @Override
    public Map<String, String> getRequestData() {
        Map<String, String> requestData = new HashMap<>();
        requestData.put("version", version);
        requestData.put("signType", signType);
        requestData.put("requestTime", requestTime);
        requestData.put("merchantAcctId", merchantAcctId);
        requestData.put("terminalId", terminalId);
        requestData.put("batchNo", batchNo);
        requestData.put("payCur", payCur);
        requestData.put("payTCnt", payTCnt);
        requestData.put("payTAmt", payTAmt);
        requestData.put("ext1", ext1);
        requestData.put("ext2", ext2);
        requestData.put("noticeUrl", noticeUrl);
        requestData.put("details", convertDetails());
        requestData.put("dupChkFlag", dupChkFlag);
        return requestData;
    }

    private String convertDetails() {
        if (details == null || details.isEmpty()) {
            return null;
        }

        StringBuffer sb = new StringBuffer();
        int size = details.size();
        for (int i=0; i<size; i++) {
            GcbAccTransferApplyRequestDetail detail = details.get(i);
            sb.append(trim(detail.getTransNo())).append("|")
                    .append(trim(detail.getOrderId())).append("|")
                    .append(trim(detail.getAmt())).append("|")
                    .append(trim(detail.getPayerAcctType())).append("|")
                    .append(trim(detail.getPayerAcctNo())).append("|")
                    .append(trim(detail.getPayeeAcctType())).append("|")
                    .append(trim(detail.getPayeeAcctNo())).append("|")
                    .append(trim(detail.getMemo())).append("|")
                    .append(trim(detail.getExt1())).append("|")
                    .append(trim(detail.getExt2()));

            if (i < (size - 1)) {
                sb.append(";");
            }
        }

        return sb.toString();
    }

    private String trim(String val) {
        if (StringUtils.isBlank(val)) {
            return StringUtils.EMPTY;
        }

        return val.trim();
    }

}