package com.huifu.jh.payout.model;

import com.huifu.jh.core.model.ParamResponse;
import org.apache.commons.lang.StringUtils;

import java.util.ArrayList;
import java.util.List;

/**
 * @author fang.wei
 * @Description 余额支付请求响应
 * @createTime 2021年10月28日 10:51:00
 */
public class GcbAccTransferApplyResponse extends ParamResponse {

    /**
     * 版本号
     */
    private String version;

    /**
     * 签名方式
     */
    private String signType;

    /**
     * 请求时间
     */
    private String requestTime;

    /**
     * 商户号
     */
    private String merchantAcctId;

    /**
     * 终端号
     */
    private String terminalId;

    /**
     * 批次号
     */
    private String batchNo;

    /**
     * 付款币种
     */
    private String payCur;

    /**
     * 总笔数
     */
    private String payTCnt;

    /**
     * 总金额
     */
    private String payTAmt;

    /**
     * 扩展字段1
     */
    private String ext1;

    /**
     * 扩展字段2
     */
    private String ext2;

    /**
     * 结果通知地址
     */
    private String noticeUrl;

    /**
     * 明细 存在错误明细时返回（只返回错误明细）,格式见返回明细字段说明○2
     */
    private String details;

    /**
     * 状态 1、受理成功 2、受理失败
     */
    private String status;

    /**
     * 错误码 格式校验未通过时返回该字段
     */
    private String errorCode;

    /**
     * 错误描述 格式校验未通过时返回该字段
     */
    private String errorMsg;

    public String getVersion() {
        return version;
    }

    public void setVersion(String version) {
        this.version = version;
    }

    public String getSignType() {
        return signType;
    }

    public void setSignType(String signType) {
        this.signType = signType;
    }

    public String getRequestTime() {
        return requestTime;
    }

    public void setRequestTime(String requestTime) {
        this.requestTime = requestTime;
    }

    public String getMerchantAcctId() {
        return merchantAcctId;
    }

    public void setMerchantAcctId(String merchantAcctId) {
        this.merchantAcctId = merchantAcctId;
    }

    public String getTerminalId() {
        return terminalId;
    }

    public void setTerminalId(String terminalId) {
        this.terminalId = terminalId;
    }

    public String getBatchNo() {
        return batchNo;
    }

    public void setBatchNo(String batchNo) {
        this.batchNo = batchNo;
    }

    public String getPayCur() {
        return payCur;
    }

    public void setPayCur(String payCur) {
        this.payCur = payCur;
    }

    public String getPayTCnt() {
        return payTCnt;
    }

    public void setPayTCnt(String payTCnt) {
        this.payTCnt = payTCnt;
    }

    public String getPayTAmt() {
        return payTAmt;
    }

    public void setPayTAmt(String payTAmt) {
        this.payTAmt = payTAmt;
    }

    public String getExt1() {
        return ext1;
    }

    public void setExt1(String ext1) {
        this.ext1 = ext1;
    }

    public String getExt2() {
        return ext2;
    }

    public void setExt2(String ext2) {
        this.ext2 = ext2;
    }

    public String getNoticeUrl() {
        return noticeUrl;
    }

    public void setNoticeUrl(String noticeUrl) {
        this.noticeUrl = noticeUrl;
    }

    public String getDetails() {
        return details;
    }

    public void setDetails(String details) {
        this.details = details;
    }

    public String getStatus() {
        return status;
    }

    public void setStatus(String status) {
        this.status = status;
    }

    public String getErrorCode() {
        return errorCode;
    }

    public void setErrorCode(String errorCode) {
        this.errorCode = errorCode;
    }

    public String getErrorMsg() {
        return errorMsg;
    }

    public void setErrorMsg(String errorMsg) {
        this.errorMsg = errorMsg;
    }

    public List<GcbAccTransferApplyResponseDetail> getDetailList() {
        if (StringUtils.isBlank(details)) {
            return null;
        }

        List<GcbAccTransferApplyResponseDetail> list = new ArrayList<>();
        String[] detailLines = details.split(";");
        for (String line : detailLines) {
            String[] fields = line.split("\\|", -1);
            GcbAccTransferApplyResponseDetail detail = new GcbAccTransferApplyResponseDetail();
            detail.setTransNo(trim(fields[0]));
            detail.setOrderId(trim(fields[1]));
            detail.setAmt(trim(fields[2]));
            detail.setPayerAcctType(trim(fields[3]));
            detail.setPayerAcctNo(fields[4]);
            detail.setPayeeAcctType(trim(fields[5]));
            detail.setPayeeAcctNo(trim(fields[6]));
            detail.setMemo(trim(fields[7]));
            detail.setExt1(trim(fields[8]));
            detail.setExt2(trim(fields[9]));

            if (fields.length > 10) {
                detail.setErrorMsg(trim(fields[10]));
            }
            list.add(detail);
        }
        return list;
    }

    private String trim(String val) {
        if (StringUtils.isBlank(val)) {
            return null;
        }

        return val.trim();
    }

    /**
     * 结果非明确失败，建议调用查询接口反查
     * @return
     */
    public PayoutRespStatus getRespStatus() {
        if (!isHttpStatusSuccess()) {
            return PayoutRespStatus.UNKNOWN;
        }

        if ("1".equals(status)) {
            return PayoutRespStatus.SUCCESS;
        }

        if ("2".equals(status)) {
            return PayoutRespStatus.FAILURE;
        }

        return PayoutRespStatus.UNKNOWN;
    }

    @Override
    public String toString() {
        return "GcbAccTransferApplyResponse{" +
                "version='" + version + '\'' +
                ", signType='" + signType + '\'' +
                ", requestTime='" + requestTime + '\'' +
                ", merchantAcctId='" + merchantAcctId + '\'' +
                ", terminalId='" + terminalId + '\'' +
                ", batchNo='" + batchNo + '\'' +
                ", payCur='" + payCur + '\'' +
                ", payTCnt='" + payTCnt + '\'' +
                ", payTAmt='" + payTAmt + '\'' +
                ", ext1='" + ext1 + '\'' +
                ", ext2='" + ext2 + '\'' +
                ", noticeUrl='" + noticeUrl + '\'' +
                ", details='" + details + '\'' +
                ", status='" + status + '\'' +
                ", errorCode='" + errorCode + '\'' +
                ", errorMsg='" + errorMsg + '\'' +
                '}';
    }

}