package com.huifu.jh.payout.model;

import com.huifu.jh.core.model.XmlRequest;
import com.huifu.jh.core.model.XmlRequestWrapper;
import com.huifu.jh.payout.model.wrapper.GcbAgcyPaymentApplyRequestWrapper;
import org.apache.commons.lang.StringUtils;

import java.util.List;

/**
 * @author fang.wei
 * @Description 商户代发请求
 * @createTime 2021年10月20日 12:32:00
 */
public class GcbAgcyPaymentApplyRequest implements XmlRequest<GcbAgcyPaymentApplyResponse> {

    /**
     * 商户号
     * 会员账号，由我司提供的商户号+01，共13位数字
     */
    private String merchantId;

    /**
     * 虚拟终端号
     * 汇付给的虚拟终端号
     */
    private String terminalId;

    /**
     * 产品类型
     * 固定为：JH
     */
    private String productType = "JH";

    /**
     * 版本号 当前版本：1.0
     */
    private String version = "1.0";

    /**
     * 交易类别
     * 100040：代发申请
     */
    private String trxType = "100040";

    /**
     * 代发批次号
     * 商户系统中唯一性ID
     */
    private String batchNo;

    /**
     * 请求时间
     * 格式：yyyyMMddHHmmss
     * 例如：20161117020101
     */
    private String requestTime;

    /**
     * 币种
     * 默认CNY
     */
    private String payCur = "CNY";

    /**
     * 总笔数
     */
    private String payTotalCnt;

    /**
     * 总金额
     * 本次请求的总金额，格式为实际金额扩大100倍提交。
     * 如10000,表示100元
     */
    private String payTotalAmt;

    /**
     * 用途
     */
    private String purpose;

    /**
     * 扩展字段1
     */
    private String ext1;

    /**
     * 扩展字段2
     */
    private String ext2;

    /**
     * 结果通知地址
     */
    private String resultUrl;

    /**
     * 校验结果通知地址
     */
    private String reviewUrl;

    /**
     * 代发明细
     */
    private List<GcbAgcyPaymentApplyRequestDetail> details;

    public void setMerchantId(String merchantId) {
        this.merchantId = merchantId;
    }

    public String getTerminalId() {
        return terminalId;
    }

    public void setTerminalId(String terminalId) {
        this.terminalId = terminalId;
    }

    public String getProductType() {
        return productType;
    }

    public void setProductType(String productType) {
        this.productType = productType;
    }

    public String getVersion() {
        return version;
    }

    public void setVersion(String version) {
        this.version = version;
    }

    public String getTrxType() {
        return trxType;
    }

    public void setTrxType(String trxType) {
        this.trxType = trxType;
    }

    public String getBatchNo() {
        return batchNo;
    }

    public void setBatchNo(String batchNo) {
        this.batchNo = batchNo;
    }

    public String getPayCur() {
        return payCur;
    }

    public void setPayCur(String payCur) {
        this.payCur = payCur;
    }

    public String getPurpose() {
        return purpose;
    }

    public void setPurpose(String purpose) {
        this.purpose = purpose;
    }

    public String getExt1() {
        return ext1;
    }

    public void setExt1(String ext1) {
        this.ext1 = ext1;
    }

    public String getExt2() {
        return ext2;
    }

    public void setExt2(String ext2) {
        this.ext2 = ext2;
    }

    public String getResultUrl() {
        return resultUrl;
    }

    public void setResultUrl(String resultUrl) {
        this.resultUrl = resultUrl;
    }

    public String getReviewUrl() {
        return reviewUrl;
    }

    public void setReviewUrl(String reviewUrl) {
        this.reviewUrl = reviewUrl;
    }

    public String getRequestTime() {
        return requestTime;
    }

    public void setRequestTime(String requestTime) {
        this.requestTime = requestTime;
    }

    public String getPayTotalCnt() {
        return payTotalCnt;
    }

    public void setPayTotalCnt(String payTotalCnt) {
        this.payTotalCnt = payTotalCnt;
    }

    public String getPayTotalAmt() {
        return payTotalAmt;
    }

    public void setPayTotalAmt(String payTotalAmt) {
        this.payTotalAmt = payTotalAmt;
    }

    public List<GcbAgcyPaymentApplyRequestDetail> getDetails() {
        return details;
    }

    public void setDetails(List<GcbAgcyPaymentApplyRequestDetail> details) {
        this.details = details;
    }

    @Override
    public String getMerchantId() {
        return merchantId;
    }

    @Override
    public Class<GcbAgcyPaymentApplyResponse> getResponseClass() {
        return GcbAgcyPaymentApplyResponse.class;
    }

    @Override
    public XmlRequestWrapper generateXmlWrapper() {
        return GcbAgcyPaymentApplyRequestWrapper.custom()
                .setMerchantId(merchantId)
                .setTerminalId(terminalId)
                .setProductType(productType)
                .setVersion(version)
                .setTrxType(trxType)
                .setBatchNo(batchNo)
                .setRequestTime(requestTime)
                .setPayCur(payCur)
                .setPayTotalCnt(payTotalCnt)
                .setPayTotalAmt(payTotalAmt)
                .setPurpose(purpose)
                .setExt1(ext1)
                .setExt2(ext2)
                .setResultUrl(resultUrl)
                .setReviewUrl(reviewUrl)
                .setDetails(buildDetailString())
                .build();
    }

    /**
     * 格式化代发明细
     * @return
     */
    private String buildDetailString() {
        if (details == null || details.isEmpty()) {
            return null;
        }

        StringBuffer sb = new StringBuffer();
        int size = details.size();
        for (int i=0; i<size; i++) {
            GcbAgcyPaymentApplyRequestDetail detail = details.get(i);
            sb.append(trim(detail.getRemitNo())).append("|")
                .append(trim(detail.getAmt())).append("|")
                .append(trim(detail.getBankId())).append("|")
                .append(trim(detail.getAcctType())).append("|")
                .append(trim(detail.getAcctNo())).append("|")
                .append(trim(detail.getPayeeName())).append("|")
                .append(trim(detail.getIdType())).append("|")
                .append(trim(detail.getIdNo())).append("|")
                .append(trim(detail.getProvince())).append("|")
                .append(trim(detail.getCity())).append("|")
                .append(trim(detail.getBranchBankName())).append("|")
                .append(trim(detail.getPurpose())).append("|")
                .append(trim(detail.getSellTrxIds())).append("|")
                .append(trim(detail.getExt1())).append("|")
                .append(trim(detail.getExt2())).append("|")
                .append(trim(detail.getCnapsCode())).append("|")
                .append(trim(detail.getRemark())).append("|")
                .append(trim(detail.getRemitType()));

            if (i < (size - 1)) {
            	//防止不同系统解析成\r\n，&#13;
                //sb.append(System.lineSeparator());
                sb.append("\n");
            }
        }

        return sb.toString();
    }

    private String trim(String val) {
        if (StringUtils.isBlank(val)) {
            return StringUtils.EMPTY;
        }

        return val.trim();
    }

}