package com.huifu.jh.payout.model;

import com.huifu.jh.core.model.XmlRequest;
import com.huifu.jh.core.model.XmlRequestWrapper;
import com.huifu.jh.payout.model.wrapper.GcbAgcyPaymentQueryRequestWrapper;

/**
 * @author fang.wei
 * @Description 商户代发结果查询请求
 * @createTime 2021年10月25日 15:30:00
 */
public class GcbAgcyPaymentQueryRequest implements XmlRequest<GcbAgcyPaymentQueryResponse> {

    /**
     * 商户号
     * 会员账号，由我司提供的商户号+01，共13位数字
     */
    private String merchantId;

    /**
     * 虚拟终端号
     * 汇付给的虚拟终端号
     */
    private String terminalId;

    /**
     * 产品类型
     * 固定为：JH
     */
    private String productType = "JH";

    /**
     * 版本号 当前版本：1.0
     */
    private String version = "1.0";

    /**
     * 交易类别
     * 100040：代发申请
     */
    private String trxType = "200030";

    /**
     * 代发批次号
     * 商户系统中唯一性ID
     */
    private String batchNo;

    /**
     * 明细流水号
     */
    private String remitReqId;

    /**
     * 源交易日期
     * 格式：yyyyMMdd
     * 例如：20180503
     */
    private String applyDate;

    public void setMerchantId(String merchantId) {
        this.merchantId = merchantId;
    }

    public String getTerminalId() {
        return terminalId;
    }

    public void setTerminalId(String terminalId) {
        this.terminalId = terminalId;
    }

    public String getProductType() {
        return productType;
    }

    public void setProductType(String productType) {
        this.productType = productType;
    }

    public String getVersion() {
        return version;
    }

    public void setVersion(String version) {
        this.version = version;
    }

    public String getTrxType() {
        return trxType;
    }

    public void setTrxType(String trxType) {
        this.trxType = trxType;
    }

    public String getBatchNo() {
        return batchNo;
    }

    public void setBatchNo(String batchNo) {
        this.batchNo = batchNo;
    }

    public String getRemitReqId() {
        return remitReqId;
    }

    public void setRemitReqId(String remitReqId) {
        this.remitReqId = remitReqId;
    }

    public String getApplyDate() {
        return applyDate;
    }

    public void setApplyDate(String applyDate) {
        this.applyDate = applyDate;
    }

    @Override
    public String getMerchantId() {
        return merchantId;
    }

    @Override
    public Class<GcbAgcyPaymentQueryResponse> getResponseClass() {
        return GcbAgcyPaymentQueryResponse.class;
    }

    @Override
    public XmlRequestWrapper generateXmlWrapper() {
        return GcbAgcyPaymentQueryRequestWrapper.custom()
                .setMerchantId(merchantId)
                .setTerminalId(terminalId)
                .setProductType(productType)
                .setVersion(version)
                .setTrxType(trxType)
                .setBatchNo(batchNo)
                .setRemitReqId(remitReqId)
                .setApplyDate(applyDate)
                .build();
    }
}