package com.huifu.jh.payout.model;

import com.huifu.jh.core.model.ParamRequest;

import java.util.HashMap;
import java.util.Map;

/**
 * @author fang.wei
 * @Description 用户提现查询请求
 * @createTime 2021年10月28日 00:01:00
 */
public class GcbUserAgcyPaymentQueryRequest implements ParamRequest<GcbUserAgcyPaymentQueryResponse> {

    /**
     * 网关版本 固定值：1
     * 必填
     */
    private String version = "1";

    /**
     * 签名类型 固定值：4。RSA加签
     * 必填
     */
    private String signType = "4";

    /**
     * 请求流水号 商户系统中唯一性ID
     * 必填
     */
    private String requestId;

    /**
     * 请求时间 yyyyMMddHHmmss
     * 必填
     */
    private String requestTime;

    /**
     * 请求会员号 会员账号，由我司提供的商户号+01，共13位数字
     * 必填
     */
    private String merchantAcctId;

    /**
     * 终端号 由我司提供
     * 必填
     */
    private String terminalId;

    /**
     * 提现批次号 商户系统中唯一标识该笔批次
     * 必填
     */
    private String batchNo;

    /**
     * 提现明细流水号 关联明细字段的唯一流水号
     * 必填
     */
    private String remitNo;

    /**
     * 原批次申请日期 为批次申请提交的日期 格式：yyyyMMdd例如：20180920
     * 必填
     */
    private String applyDate;

    public String getVersion() {
        return version;
    }

    public void setVersion(String version) {
        this.version = version;
    }

    public String getSignType() {
        return signType;
    }

    public void setSignType(String signType) {
        this.signType = signType;
    }

    public String getRequestId() {
        return requestId;
    }

    public void setRequestId(String requestId) {
        this.requestId = requestId;
    }

    public String getRequestTime() {
        return requestTime;
    }

    public void setRequestTime(String requestTime) {
        this.requestTime = requestTime;
    }

    public String getMerchantAcctId() {
        return merchantAcctId;
    }

    public void setMerchantAcctId(String merchantAcctId) {
        this.merchantAcctId = merchantAcctId;
    }

    public String getTerminalId() {
        return terminalId;
    }

    public void setTerminalId(String terminalId) {
        this.terminalId = terminalId;
    }

    public String getBatchNo() {
        return batchNo;
    }

    public void setBatchNo(String batchNo) {
        this.batchNo = batchNo;
    }

    public String getRemitNo() {
        return remitNo;
    }

    public void setRemitNo(String remitNo) {
        this.remitNo = remitNo;
    }

    public String getApplyDate() {
        return applyDate;
    }

    public void setApplyDate(String applyDate) {
        this.applyDate = applyDate;
    }

    @Override
    public String getMerchantId() {
        return merchantAcctId;
    }

    @Override
    public Class<GcbUserAgcyPaymentQueryResponse> getResponseClass() {
        return GcbUserAgcyPaymentQueryResponse.class;
    }

    @Override
    public String[] getSignParamNames() {
        return new String[] {"applyDate","batchNo","merchantAcctId","remitNo","requestId","requestTime","signType","terminalId","version"};
    }

    @Override
    public Map<String, String> getRequestData() {
        Map<String, String> requestData = new HashMap<>();
        requestData.put("version", version);
        requestData.put("signType", signType);
        requestData.put("requestId", requestId);
        requestData.put("requestTime", requestTime);
        requestData.put("merchantAcctId", merchantAcctId);
        requestData.put("terminalId", terminalId);
        requestData.put("batchNo", batchNo);
        requestData.put("remitNo", remitNo);
        requestData.put("applyDate", applyDate);
        return requestData;
    }
}