package com.huifu.jh.payout.model.wrapper;

import com.huifu.jh.core.model.XmlRequestWrapper;

import javax.xml.bind.annotation.XmlAccessType;
import javax.xml.bind.annotation.XmlAccessorType;
import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlRootElement;

/**
 * @author fang.wei
 * @Description 商户代发结果查询请求报文封装
 * @createTime 2021年10月25日 15:35:00
 */
@XmlRootElement(name = "REQUEST")
@XmlAccessorType(XmlAccessType.FIELD)
public class GcbAgcyPaymentQueryRequestWrapper implements XmlRequestWrapper<GcbAgcyPaymentQueryResponseWrapper> {

    @XmlElement(name = "GENERAL")
    private RequestHead head;

    @XmlElement(name = "PARAMS")
    private RequestParam params;

    public RequestHead getHead() {
        return head;
    }

    public void setHead(RequestHead head) {
        this.head = head;
    }

    public RequestParam getParams() {
        return params;
    }

    public void setParams(RequestParam params) {
        this.params = params;
    }

    public static GcbAgcyPaymentQueryRequestWrapper.Builder custom() {
        return new Builder();
    }

    @Override
    public Class<GcbAgcyPaymentQueryResponseWrapper> getResponseWrapperClass() {
        return GcbAgcyPaymentQueryResponseWrapper.class;
    }

    public static class Builder {

        private String merchantId;
        private String terminalId;
        private String productType;
        private String version;
        private String trxType;
        private String batchNo;
        private String remitReqId;
        private String applyDate;

        public Builder setMerchantId(String merchantId) {
            this.merchantId = merchantId;
            return this;
        }

        public Builder setTerminalId(String terminalId) {
            this.terminalId = terminalId;
            return this;
        }

        public Builder setProductType(String productType) {
            this.productType = productType;
            return this;
        }

        public Builder setVersion(String version) {
            this.version = version;
            return this;
        }

        public Builder setTrxType(String trxType) {
            this.trxType = trxType;
            return this;
        }

        public Builder setBatchNo(String batchNo) {
            this.batchNo = batchNo;
            return this;
        }

        public Builder setRemitReqId(String remitReqId) {
            this.remitReqId = remitReqId;
            return this;
        }

        public Builder setApplyDate(String applyDate) {
            this.applyDate = applyDate;
            return this;
        }

        public GcbAgcyPaymentQueryRequestWrapper build() {
            GcbAgcyPaymentQueryRequestWrapper wrapper = new GcbAgcyPaymentQueryRequestWrapper();
            wrapper.setHead(new RequestHead(merchantId, terminalId, productType, version, trxType));
            wrapper.setParams(new RequestParam(batchNo, remitReqId, applyDate));
            return wrapper;
        }
    }

    @XmlAccessorType(XmlAccessType.FIELD)
    static class RequestHead {

        public RequestHead(String merchantId, String terminalId, String productType, String version, String trxType) {
            this.merchantId = merchantId;
            this.terminalId = terminalId;
            this.productType = productType;
            this.version = version;
            this.trxType = trxType;
        }

        @XmlElement(name = "MERCHANTID")
        private String merchantId;

        @XmlElement(name = "TERMINALID")
        private String terminalId;

        @XmlElement(name = "PRODUCTTYPE")
        private String productType;

        @XmlElement(name = "VERSION")
        private String version;

        @XmlElement(name = "TRXTYPE")
        private String trxType;

        public String getProductType() {
            return productType;
        }

        public void setProductType(String productType) {
            this.productType = productType;
        }

        public String getVersion() {
            return version;
        }

        public void setVersion(String version) {
            this.version = version;
        }

        public String getTrxType() {
            return trxType;
        }

        public void setTrxType(String trxType) {
            this.trxType = trxType;
        }

        public String getMerchantId() {
            return merchantId;
        }

        public void setMerchantId(String merchantId) {
            this.merchantId = merchantId;
        }

        public String getTerminalId() {
            return terminalId;
        }

        public void setTerminalId(String terminalId) {
            this.terminalId = terminalId;
        }
    }

    @XmlAccessorType(XmlAccessType.FIELD)
    static class RequestParam {

        public RequestParam(String batchNo, String remitReqId, String applyDate) {
            this.batchNo = batchNo;
            this.remitReqId = remitReqId;
            this.applyDate = applyDate;
        }

        @XmlElement(name = "BATCH_NO")
        private String batchNo;

        @XmlElement(name = "REMIT_REQ_ID")
        private String remitReqId;

        @XmlElement(name = "APPLY_DATE")
        private String applyDate;

        public String getBatchNo() {
            return batchNo;
        }

        public void setBatchNo(String batchNo) {
            this.batchNo = batchNo;
        }

        public String getRemitReqId() {
            return remitReqId;
        }

        public void setRemitReqId(String remitReqId) {
            this.remitReqId = remitReqId;
        }

        public String getApplyDate() {
            return applyDate;
        }

        public void setApplyDate(String applyDate) {
            this.applyDate = applyDate;
        }
    }
}