package com.huifu.jh.payout.model.wrapper;

import com.huifu.jh.core.model.XmlCallbackMessageWrapper;
import com.huifu.jh.payout.model.GcbAgcyPaymentResult;
import com.huifu.jh.payout.model.GcbAgcyPaymentResultDetail;
import org.apache.commons.lang.StringUtils;

import javax.xml.bind.annotation.XmlAccessType;
import javax.xml.bind.annotation.XmlAccessorType;
import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlRootElement;
import java.util.ArrayList;
import java.util.List;

/**
 * @author fang.wei
 * @Description 商户代发异步通知结果封装类
 * @createTime 2021年10月22日 17:23:00
 */
@XmlRootElement(name = "NOTIFY")
@XmlAccessorType(XmlAccessType.FIELD)
public class GcbAgcyPaymentResultWrapper implements XmlCallbackMessageWrapper<GcbAgcyPaymentResult> {

    @XmlElement(name = "GENERAL")
    private NotifyHead head;

    @XmlElement(name = "PARAMS")
    private NotifyParam param;

    @Override
    public GcbAgcyPaymentResult toMessage() {
        GcbAgcyPaymentResult result = new GcbAgcyPaymentResult();
        result.setMerchantId(head.getMerchantId());
        result.setTerminalId(head.getTerminalId());
        result.setProductType(head.getProductType());
        result.setVersion(head.getVersion());
        result.setTrxType(head.getTrxType());
        result.setBatchNo(param.getBatchNo());
        result.setRequestTime(param.getRequestTime());
        result.setPayCur(param.getPayCur());
        result.setPayTotalCnt(param.getPayTotalCnt());
        result.setPayTotalAmt(param.getPayTotalAmt());
        result.setPurpose(param.getPurpose());
        result.setExt1(param.getExt1());
        result.setExt2(param.getExt2());
        result.setResultUrl(param.getResultUrl());
        result.setDetails(convertDetails(param.getDetails()));
        result.setStatus(param.getStatus());
        result.setErrorCode(param.getErrorCode());
        result.setErrorMsg(param.getErrorMsg());
        return result;
    }

    private List<GcbAgcyPaymentResultDetail> convertDetails(String details) {
        if (StringUtils.isBlank(details)) {
            return null;
        }

        List<GcbAgcyPaymentResultDetail> list = new ArrayList<>();
        String[] detailLines = details.split("\\\n");
        for (String line : detailLines) {
            String[] fields = line.split("\\|", -1);
            GcbAgcyPaymentResultDetail detail = new GcbAgcyPaymentResultDetail();
            detail.setRemitNo(trim(fields[0]));
            detail.setAmt(trim(fields[1]));
            detail.setBankId(trim(fields[2]));
            detail.setAcctType(trim(fields[3]));
            detail.setAcctNo(trim(fields[4]));
            detail.setPayeeName(trim(fields[5]));
            detail.setIdType(trim(fields[6]));
            detail.setIdNo(trim(fields[7]));
            detail.setProvince(trim(fields[8]));
            detail.setCity(trim(fields[9]));
            detail.setBranchBankName(trim(fields[10]));
            detail.setPurpose(trim(fields[11]));
            detail.setSellTrxIds(trim(fields[12]));
            detail.setExt1(trim(fields[13]));
            detail.setExt2(trim(fields[14]));

            if (fields.length > 15) {
                detail.setErrorMsg(trim(fields[15]));
            }

            list.add(detail);
        }

        return list;
    }

    private String trim(String val) {
        if (StringUtils.isBlank(val)) {
            return null;
        }

        return val.trim();
    }

    @XmlAccessorType(XmlAccessType.FIELD)
    static class NotifyHead {

        @XmlElement(name = "MERCHANTID")
        private String merchantId;

        @XmlElement(name = "TERMINALID")
        private String terminalId;

        @XmlElement(name = "PRODUCTTYPE")
        private String productType;

        @XmlElement(name = "VERSION")
        private String version;

        @XmlElement(name = "TRXTYPE")
        private String trxType;

        public String getProductType() {
            return productType;
        }

        public void setProductType(String productType) {
            this.productType = productType;
        }

        public String getVersion() {
            return version;
        }

        public void setVersion(String version) {
            this.version = version;
        }

        public String getTrxType() {
            return trxType;
        }

        public void setTrxType(String trxType) {
            this.trxType = trxType;
        }

        public String getMerchantId() {
            return merchantId;
        }

        public void setMerchantId(String merchantId) {
            this.merchantId = merchantId;
        }

        public String getTerminalId() {
            return terminalId;
        }

        public void setTerminalId(String terminalId) {
            this.terminalId = terminalId;
        }
    }

    @XmlAccessorType(XmlAccessType.FIELD)
    static class NotifyParam {

        @XmlElement(name = "BATCH_NO")
        private String batchNo;

        @XmlElement(name = "REQTIME")
        private String requestTime;

        @XmlElement(name = "PAY_CUR")
        private String payCur;

        @XmlElement(name = "PAY_TCNT")
        private String payTotalCnt;

        @XmlElement(name = "PAY_TAMT")
        private String payTotalAmt;

        @XmlElement(name = "PURPOSE")
        private String purpose;

        @XmlElement(name = "EXT1")
        private String ext1;

        @XmlElement(name = "EXT2")
        private String ext2;

        @XmlElement(name = "RESULT_URL")
        private String resultUrl;

        @XmlElement(name = "DETAILS")
        private String details;

        @XmlElement(name = "STATUS")
        private String status;

        @XmlElement(name = "ERRERCODE")
        private String errorCode;

        @XmlElement(name = "ERRERMSG")
        private String errorMsg;

        public String getBatchNo() {
            return batchNo;
        }

        public void setBatchNo(String batchNo) {
            this.batchNo = batchNo;
        }

        public String getRequestTime() {
            return requestTime;
        }

        public void setRequestTime(String requestTime) {
            this.requestTime = requestTime;
        }

        public String getPayTotalCnt() {
            return payTotalCnt;
        }

        public void setPayTotalCnt(String payTotalCnt) {
            this.payTotalCnt = payTotalCnt;
        }

        public String getPayTotalAmt() {
            return payTotalAmt;
        }

        public void setPayTotalAmt(String payTotalAmt) {
            this.payTotalAmt = payTotalAmt;
        }

        public String getPayCur() {
            return payCur;
        }

        public void setPayCur(String payCur) {
            this.payCur = payCur;
        }

        public String getPurpose() {
            return purpose;
        }

        public void setPurpose(String purpose) {
            this.purpose = purpose;
        }

        public String getExt1() {
            return ext1;
        }

        public void setExt1(String ext1) {
            this.ext1 = ext1;
        }

        public String getExt2() {
            return ext2;
        }

        public void setExt2(String ext2) {
            this.ext2 = ext2;
        }

        public String getResultUrl() {
            return resultUrl;
        }

        public void setResultUrl(String resultUrl) {
            this.resultUrl = resultUrl;
        }

        public String getDetails() {
            return details;
        }

        public void setDetails(String details) {
            this.details = details;
        }

        public String getStatus() {
            return status;
        }

        public void setStatus(String status) {
            this.status = status;
        }

        public String getErrorCode() {
            return errorCode;
        }

        public void setErrorCode(String errorCode) {
            this.errorCode = errorCode;
        }

        public String getErrorMsg() {
            return errorMsg;
        }

        public void setErrorMsg(String errorMsg) {
            this.errorMsg = errorMsg;
        }
    }

}