package demo.payout.controller;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Random;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.ResponseBody;

import com.alibaba.fastjson.JSON;
import com.huifu.jh.core.DefaultGcbPayCallbackMessageParser;
import com.huifu.jh.core.DefaultGcbPayClient;
import com.huifu.jh.core.GcbPayCallbackMessageParser;
import com.huifu.jh.core.GcbPayClient;
import com.huifu.jh.payout.model.GcbAccTransferApplyRequest;
import com.huifu.jh.payout.model.GcbAccTransferApplyRequestDetail;
import com.huifu.jh.payout.model.GcbAccTransferApplyResponse;
import com.huifu.jh.payout.model.GcbAccTransferApplyResult;
import com.huifu.jh.payout.model.PayoutRespStatus;

import demo.payout.config.AppConfig;

/**
 * @author fang.wei
 * @Description 余额支付DEMO
 * @createTime 2021年10月30日 23:47:00
 */
@Controller
public class GcbAccTransferApplyController {
    private Logger logger = LoggerFactory.getLogger(getClass());

    @Autowired
    private AppConfig appConfig;

    @GetMapping("/showGcbAccTransferApply.do")
    public String showGcbAccTransferApply(HttpServletRequest request1, HttpServletResponse response, Model model) {
        // 构建商户代发请求对象
        GcbAccTransferApplyRequest request = new GcbAccTransferApplyRequest();
        //必填，请求时间 yyyyMMddHHmmss
        request.setRequestTime(new SimpleDateFormat("yyyyMMddHHmmss").format(new Date()));
        //必填，会员账号，由我司提供的商户号+01，共13位数字
        request.setMerchantAcctId("1001215986501");
        //必填，终端号，由我司提供的终端号
        request.setTerminalId("JHDF002");
        //必填，代发批次号 商户系统唯一ID
        request.setBatchNo(new SimpleDateFormat("yyyyMMddHHmmssSSS").format(new Date()));
        //必填，明细总笔数
        request.setPayTCnt("2");
        //必填，疑似重复标志  默认1-校验
        request.setDupChkFlag("1");
        //非必填，扩展字段1
        request.setExt1("");
        //非必填，扩展字段2
        request.setExt2("");
        //必填，结果通知地址
        request.setNoticeUrl("http://localhost:8080/receiveTransferResult.do");
        //必填，代发明细
        request.setDetails(mockDetails());

        List<GcbAccTransferApplyRequestDetail> details = mockDetails();
        //必填，代发明细
        request.setDetails(details);

        //必填，请求总金额 格式为实际金额扩大100倍提交。 如10000,表示100元
        request.setPayTAmt(sumAmt(details));

        model.addAttribute("req", request);
        model.addAttribute("jsonDetails", JSON.toJSONString(request.getDetails()));
        return "GcbAccTransferApply";
    }

    private String sumAmt(List<GcbAccTransferApplyRequestDetail> details) {
        int sumAmt = 0;
        for (GcbAccTransferApplyRequestDetail d : details) {
            sumAmt += (Integer.parseInt(d.getAmt()));
        }
        return String.valueOf(sumAmt);
    }

    private static List<GcbAccTransferApplyRequestDetail> mockDetails() {
        List<GcbAccTransferApplyRequestDetail> list = new ArrayList<>();
        GcbAccTransferApplyRequestDetail d = new GcbAccTransferApplyRequestDetail();
        d.setTransNo(new SimpleDateFormat("yyyyMMddHHmmssSSS").format(new Date()));
        d.setOrderId(new SimpleDateFormat("yyyyMMddHHmmssSSS").format(new Date()));
        d.setAmt("" + new Random().nextInt(1000));
        d.setPayerAcctType("M");
        d.setPayerAcctNo("10012159865");
        d.setPayeeAcctType("H");
        d.setPayeeAcctNo("10000001");
        d.setMemo("付款背景");
        list.add(d);

        GcbAccTransferApplyRequestDetail d1 = new GcbAccTransferApplyRequestDetail();
        d1.setTransNo(new SimpleDateFormat("yyyyMMddHHmmssSSS").format(new Date()) + "1");
        d1.setOrderId(new SimpleDateFormat("yyyyMMddHHmmssSSS").format(new Date()) + "1");
        d1.setAmt("" + new Random().nextInt(1000));
        d1.setPayerAcctType("M");
        d1.setPayerAcctNo("10012159865");
        d1.setPayeeAcctType("H");
        d1.setPayeeAcctNo("10000001");
        d1.setMemo("付款背景");
        list.add(d1);
        return list;
    }

    @PostMapping("/doGcbAccTransferApply.do")
    @ResponseBody
    public String doGcbAccTransferApply(HttpServletRequest request, HttpServletResponse response, Model model, GcbAccTransferApplyRequest req) throws Exception {

        // 服务url(测试环境)
        String serverUrl = "https://hfgj.testpnr.com/acctransfer/transfer.htm";
        // 服务url(生产环境)
        //	String serverUrl = "https://global.chinapnr.com/acctransfer/transfer.htm";

        // 实例化客户端
        GcbPayClient gcbPayClient = new DefaultGcbPayClient(serverUrl, appConfig.getBase64PrivateKey(), appConfig.getBase64PublicKey());

        String details = request.getParameter("jsonDetails");
        req.setDetails(JSON.parseArray(details, GcbAccTransferApplyRequestDetail.class));

        // 发送请求
        GcbAccTransferApplyResponse resp = gcbPayClient.execute(req);
        logger.info("resp => " + new String(resp.getResponseBody()));

        // TODO 业务处理
        // ......

        String respTxt = "";
        PayoutRespStatus status = resp.getRespStatus();
        switch (status) {
            case FAILURE:
                respTxt = "请求失败" + "<br />";
                break;
            case SUCCESS:
                respTxt = "请求成功" + "<br />";
                break;
            case UNKNOWN:
                respTxt = "请求结果未知，需反查" + "<br />";
                // TODO 调用结果查询
        }

        respTxt += resp.toString();
        return respTxt;
    }

    @PostMapping("/receiveTransferResult.do")
    public String receiveTransferResult(HttpServletRequest request, HttpServletResponse response, Model model) {
        // 实例化通知解析器
        GcbPayCallbackMessageParser gcbMessageParser = new DefaultGcbPayCallbackMessageParser(appConfig.getBase64PublicKey());

        try {
            GcbAccTransferApplyResult result = gcbMessageParser.parse(request, GcbAccTransferApplyResult.class);
            logger.info("result -> " + JSON.toJSONString(result));

            // 向服务端返回接收成功
            response.getWriter().write("<result>000000</result>");
        } catch (Exception e) {
            e.printStackTrace();
            // TODO
        }

        return null;
    }

}