package demo.payout.controller;

import java.text.SimpleDateFormat;
import java.util.Date;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.ResponseBody;

import com.huifu.jh.core.DefaultGcbPayClient;
import com.huifu.jh.core.GcbPayClient;
import com.huifu.jh.payout.model.GcbAccTransferQueryRequest;
import com.huifu.jh.payout.model.GcbAccTransferQueryResponse;

import demo.payout.config.AppConfig;

/**
 * @author fang.wei
 * @Description 余额支付结果查询DEMO
 * @createTime 2021年10月30日 23:47:00
 */
@Controller
public class GcbAccTransferQueryController {
    private Logger logger = LoggerFactory.getLogger(getClass());

    @Autowired
    private AppConfig appConfig;

    @GetMapping("/showGcbAccTransferQuery.do")
    public String showGcbAccTransferQuery(HttpServletRequest request1, HttpServletResponse response, Model model) {
        // 构建商户代发请求对象
        GcbAccTransferQueryRequest request = new GcbAccTransferQueryRequest();
        //必填，会员账号，由我司提供的商户号+01，共13位数字
        request.setMerchantAcctId("1001215986501");
        //必填，终端号，由我司提供的终端号
        request.setTerminalId("JHDF002");
        String randomNum = new SimpleDateFormat("yyyyMMddHHmmssSSS").format(new Date());
        String time = new SimpleDateFormat("yyyyMMddHHmmss").format(new Date());
        request.setRequestId(randomNum);
        request.setRequestTime(time);

        //必填，批次号 商户系统唯一ID
        request.setOrigbatchNo("20211029160920002");
        //必填，明细流水号
        request.setOrigtransNo("202110291609200041");
        //必填，明细总笔数
        request.setOrigApplyDate("20211029");

        model.addAttribute("req", request);
        return "GcbAccTransferQuery";
    }

    @PostMapping("/doGcbAccTransferQuery.do")
    @ResponseBody
    public String doGcbAccTransferQuery(HttpServletRequest request, HttpServletResponse response, Model model, GcbAccTransferQueryRequest req) throws Exception {

        // 服务url(测试环境)
        String serverUrl = "https://hfgj.testpnr.com/acctransfer/transferQuery.htm";
        // 服务url(生产环境)
        //	String serverUrl = "https://global.chinapnr.com/acctransfer/transferQuery.htm";

        // 实例化客户端
        GcbPayClient gcbPayClient = new DefaultGcbPayClient(serverUrl, appConfig.getBase64PrivateKey(), appConfig.getBase64PublicKey());

        // 发送请求
        GcbAccTransferQueryResponse resp = gcbPayClient.execute(req);
        logger.info("resp => " + new String(resp.getResponseBody()));

        // TODO 业务处理
        // ......

        String respTxt = "响应信息<br />" + resp.toString();
        return respTxt;
    }


}