package demo.payout.controller;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Random;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.ResponseBody;

import com.alibaba.fastjson.JSON;
import com.huifu.jh.core.DefaultGcbPayCallbackMessageParser;
import com.huifu.jh.core.DefaultGcbPayClient;
import com.huifu.jh.core.GcbPayCallbackMessageParser;
import com.huifu.jh.core.GcbPayClient;
import com.huifu.jh.core.model.XmlCallbackResponse;
import com.huifu.jh.payout.model.GcbAgcyPaymentApplyRequest;
import com.huifu.jh.payout.model.GcbAgcyPaymentApplyRequestDetail;
import com.huifu.jh.payout.model.GcbAgcyPaymentApplyResponse;
import com.huifu.jh.payout.model.GcbAgcyPaymentResult;
import com.huifu.jh.payout.model.GcbAgcyPaymentResultResponse;
import com.huifu.jh.payout.model.PayoutRespStatus;

import demo.payout.config.AppConfig;

/**
 * @author fang.wei
 * @Description 商户代发DEMO
 * @createTime 2021年10月30日 23:47:00
 */
@Controller
public class GcbAgcyPaymentApplyController {
    private Logger logger = LoggerFactory.getLogger(getClass());

    @Autowired
    private AppConfig appConfig;

    @GetMapping("/showGcbAgcyPaymentApply.do")
    public String showGcbAgcyPaymentApply(HttpServletRequest request1, HttpServletResponse response, Model model) {
        // 构建商户代发请求对象
        GcbAgcyPaymentApplyRequest request = new GcbAgcyPaymentApplyRequest();
        //必填，会员账号，由我司提供的商户号+01，共13位数字
        request.setMerchantId("1001215986501");
        //必填，终端号，由我司提供的终端号
        request.setTerminalId("JHDF002");
        //必填，代发批次号 商户系统唯一ID
        request.setBatchNo(new SimpleDateFormat("yyyyMMddHHmmssSSS").format(new Date()));
        //必填，请求时间 yyyyMMddHHmmss
        request.setRequestTime(new SimpleDateFormat("yyyyMMddHHmmss").format(new Date()));
        //必填，明细总笔数
        request.setPayTotalCnt("2");
        //必填，用途
        request.setPurpose("工资");
        //非必填，扩展字段1
        request.setExt1("");
        //非必填，扩展字段2
        request.setExt2("");
        //必填，结果通知地址
        request.setReviewUrl("http://localhost:8080/receiveAgcyPaymentResult.do");
        //必填，审核通知地址
        request.setResultUrl("http://localhost:8080/receiveAgcyPaymentResult.do");

        List<GcbAgcyPaymentApplyRequestDetail> details = mockDetails();
        //必填，代发明细
        request.setDetails(details);

        //必填，请求总金额 格式为实际金额扩大100倍提交。 如10000,表示100元
        request.setPayTotalAmt(sumAmt(details));

        model.addAttribute("req", request);
        model.addAttribute("jsonDetails", JSON.toJSONString(request.getDetails()));
        return "GcbAgcyPaymentApply";
    }

    private String sumAmt(List<GcbAgcyPaymentApplyRequestDetail> details) {
        int sumAmt = 0;
        for (GcbAgcyPaymentApplyRequestDetail d : details) {
            sumAmt += (Integer.parseInt(d.getAmt()));
        }
        return String.valueOf(sumAmt);
    }

    private List<GcbAgcyPaymentApplyRequestDetail> mockDetails() {
        List<GcbAgcyPaymentApplyRequestDetail> list = new ArrayList<>();
        GcbAgcyPaymentApplyRequestDetail detail = newDetail();
        list.add(detail);

        GcbAgcyPaymentApplyRequestDetail detail1 = newDetail();
        list.add(detail1);
        return list;
    }

    private GcbAgcyPaymentApplyRequestDetail newDetail() {
        GcbAgcyPaymentApplyRequestDetail detail = new GcbAgcyPaymentApplyRequestDetail();
        detail.setRemitNo(new SimpleDateFormat("yyyyMMddHHmmssSSS").format(new Date()) + new Random().nextInt(1000));
        detail.setAmt("" + new Random().nextInt(1000));
        detail.setBankId("CMB");
        detail.setAcctType("P");
        detail.setAcctNo("1111111111");
        detail.setPayeeName("李财水");
        detail.setIdType("01");
        detail.setIdNo("320125198805232313");
        detail.setProvince("江苏省");
        detail.setCity("南京市");
        detail.setBranchBankName("招行银行南京分行城东支行");
        detail.setPurpose("Test");
        detail.setSellTrxIds("T1001");
        detail.setExt1("ext1");
        detail.setExt2("ext2");
        detail.setCnapsCode("");
        detail.setRemark("备注");
        detail.setRemitType("");
        return detail;
    }

    @PostMapping("/doGcbAgcyPaymentApply.do")
    @ResponseBody
    public String doGcbAgcyPaymentApply(HttpServletRequest request, HttpServletResponse response, Model model, GcbAgcyPaymentApplyRequest req) throws Exception {

        // 服务url(测试环境)
        String serverUrl = "https://hfgj.testpnr.com/sell/receivesell.htm";
        // 服务url(生产环境)
        //	String serverUrl = "https://global.chinapnr.com/sell/receivesell.htm";

        // 实例化客户端
        GcbPayClient gcbPayClient = new DefaultGcbPayClient(serverUrl, appConfig.getBase64PrivateKey(), appConfig.getBase64PublicKey());

        String details = request.getParameter("jsonDetails");
        req.setDetails(JSON.parseArray(details, GcbAgcyPaymentApplyRequestDetail.class));

        // 发送请求
        GcbAgcyPaymentApplyResponse resp = gcbPayClient.execute(req);
        logger.info("resp => " + new String(resp.getResponseBody()));

        // TODO 业务处理
        // ......

        String respTxt = "";
        PayoutRespStatus status = resp.getRespStatus();
        switch (status) {
            case FAILURE:
                respTxt = "请求失败" + "<br />";
                break;
            case SUCCESS:
                respTxt = "请求成功" + "<br />";
                break;
            case UNKNOWN:
                respTxt = "请求结果未知，需反查" + "<br />";
                // TODO 调用结果查询
        }

        respTxt += resp.toString();
        return respTxt;
    }

    @PostMapping("/receiveAgcyPaymentResult.do")
    public String receiveApplyResult(HttpServletRequest request, HttpServletResponse response, Model model) {
        // 实例化通知解析器
        GcbPayCallbackMessageParser gcbMessageParser = new DefaultGcbPayCallbackMessageParser(appConfig.getBase64PublicKey());

        try {
            GcbAgcyPaymentResult result = gcbMessageParser.parse(request, GcbAgcyPaymentResult.class);
            logger.info("result -> " + JSON.toJSONString(result));

            // 向服务端返回接收成功
            XmlCallbackResponse resp = buildResponse(result);
            response.getWriter().write(resp.toXml());
        } catch (Exception e) {
            e.printStackTrace();
            // TODO
        }

        return null;
    }

    private GcbAgcyPaymentResultResponse buildResponse(GcbAgcyPaymentResult result) {
        GcbAgcyPaymentResultResponse resp = new GcbAgcyPaymentResultResponse();
        resp.setMerchantId(result.getMerchantId());
        resp.setTerminalId(result.getTerminalId());
        resp.setProductType(result.getProductType());
        resp.setVersion(result.getVersion());
        resp.setTrxType(result.getTrxType());
        resp.setResult("000000");
        return resp;
    }

}