package demo.payout.controller;

import com.alibaba.fastjson.JSON;
import com.huifu.jh.core.DefaultGcbPayCallbackMessageParser;
import com.huifu.jh.core.DefaultGcbPayClient;
import com.huifu.jh.core.GcbPayCallbackMessageParser;
import com.huifu.jh.core.GcbPayClient;
import com.huifu.jh.payout.model.*;
import demo.payout.config.AppConfig;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.ResponseBody;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Random;

/**
 * @author fang.wei
 * @Description 用户提现DEMO
 * @createTime 2021年10月30日 23:47:00
 */
@Controller
public class GcbUserAgcyPaymentApplyController {
    private Logger logger = LoggerFactory.getLogger(getClass());

    @Autowired
    private AppConfig appConfig;

    @GetMapping("/showGcbUserAgcyPaymentApply.do")
    public String showGcbAgcyPaymentApply(HttpServletRequest request1, HttpServletResponse response, Model model) {

        GcbUserAgcyPaymentApplyRequest request = new GcbUserAgcyPaymentApplyRequest();

        String randomNum = new SimpleDateFormat("yyyyMMddHHmmssSSS").format(new Date());
        String time = new SimpleDateFormat("yyyyMMddHHmmss").format(new Date());
        request.setRequestId(randomNum);
        request.setRequestTime(time);
        request.setMerchantAcctId("1001215986501");
        request.setTerminalId("JHDF002");
        request.setBatchNo(randomNum);
        request.setPayTCnt("2");
        request.setIgnoreFlag("1");
        request.setPurpose("用途");
        request.setNoticeUrl("http://localhost:8080/receiveUserAgcyPaymentResult.do");

        List<GcbUserAgcyPaymentApplyRequestDetail> details = mockDetails();
        //必填，代发明细
        request.setDetails(details);

        //必填，请求总金额 格式为实际金额扩大100倍提交。 如10000,表示100元
        request.setPayTAmt(sumAmt(details));

        model.addAttribute("req", request);
        model.addAttribute("jsonDetails", JSON.toJSONString(request.getDetails()));
        return "GcbUserAgcyPaymentApply";
    }

    private List<GcbUserAgcyPaymentApplyRequestDetail> mockDetails() {
        List<GcbUserAgcyPaymentApplyRequestDetail> list = new ArrayList<>();
        GcbUserAgcyPaymentApplyRequestDetail d = new GcbUserAgcyPaymentApplyRequestDetail();
        String randomNum = new SimpleDateFormat("yyyyMMddHHmmssSSS").format(new Date());
        d.setRemitNo(randomNum);
        d.setAmt("" + new Random().nextInt(1000));
        d.setBankId("CMB");
        d.setAcctType("P");
        d.setRemitType("1");
        d.setAcctNo("111111");
        d.setPurpose("用途");
        d.setPayeeName("李财水");
        d.setIdType("00");
        d.setIdNo("320125198805232313");
        d.setRemark("工资代发");
        d.setUserId("10000001");
        list.add(d);

        GcbUserAgcyPaymentApplyRequestDetail d1 = new GcbUserAgcyPaymentApplyRequestDetail();
        d1.setRemitNo(randomNum + "1");
        d1.setAmt("" + new Random().nextInt(1000));
        d1.setBankId("CMB");
        d1.setAcctType("P");
        d1.setRemitType("1");
        d1.setAcctNo("111111");
        d1.setPurpose("用途");
        d1.setPayeeName("李财水");
        d1.setIdType("00");
        d1.setIdNo("320125198805232313");
        d1.setRemark("工资代发");
        d1.setUserId("10000001");
        list.add(d1);
        return list;
    }

    private String sumAmt(List<GcbUserAgcyPaymentApplyRequestDetail> details) {
        int sumAmt = 0;
        for (GcbUserAgcyPaymentApplyRequestDetail d : details) {
            sumAmt += (Integer.parseInt(d.getAmt()));
        }
        return String.valueOf(sumAmt);
    }

    @PostMapping("/doGcbUserAgcyPaymentApply.do")
    @ResponseBody
    public String doGcbUserAgcyPaymentApply(HttpServletRequest request, HttpServletResponse response, Model model, GcbUserAgcyPaymentApplyRequest req) throws Exception {

        // 服务url(测试环境)
        String serverUrl = "https://hfgj.testpnr.com/mcs/remit.htm";
        // 服务url(生产环境)
        //	String serverUrl = "https://global.chinapnr.com/mcs/remit.htm";

        // 实例化客户端
        GcbPayClient gcbPayClient = new DefaultGcbPayClient(serverUrl, appConfig.getBase64PrivateKey(), appConfig.getBase64PublicKey());

        String details = request.getParameter("jsonDetails");
        req.setDetails(JSON.parseArray(details, GcbUserAgcyPaymentApplyRequestDetail.class));

        // 发送请求
        GcbUserAgcyPaymentApplyResponse resp = gcbPayClient.execute(req);
        logger.info("resp => " + new String(resp.getResponseBody()));

        // TODO 业务处理
        // ......

        String respTxt = "";
        PayoutRespStatus status = resp.getRespStatus();
        switch (status) {
            case FAILURE:
                respTxt = "请求失败" + "<br />";
                break;
            case SUCCESS:
                respTxt = "请求成功" + "<br />";
                break;
            case PART_SUCCESS:
                respTxt = "部分成功" + "<br />";
                break;
            case UNKNOWN:
                respTxt = "请求结果未知，需反查" + "<br />";
                // TODO 调用结果查询
        }

        respTxt += resp.toString();
        return respTxt;
    }

    @PostMapping("/receiveUserAgcyPaymentResult.do")
    public String receiveUserAgcyPaymentResult(HttpServletRequest request, HttpServletResponse response, Model model) {
        // 实例化通知解析器
        GcbPayCallbackMessageParser gcbMessageParser = new DefaultGcbPayCallbackMessageParser(appConfig.getBase64PublicKey());

        try {
            GcbUserAgcyPaymentResult result = gcbMessageParser.parse(request, GcbUserAgcyPaymentResult.class);
            logger.info("result -> " + JSON.toJSONString(result));

            // 向服务端返回接收成功
            response.getWriter().write("<result>000000</result>");
        } catch (Exception e) {
            e.printStackTrace();
            // TODO
        }

        return null;
    }

}